package ch.psi.daq.test.queryrest.controller;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

import java.util.List;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.junit.After;
import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.bsread.message.Type;
import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.common.time.TimeUtils;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.json.channels.info.ChannelInfo;
import ch.psi.daq.domain.json.channels.info.ChannelInfos;
import ch.psi.daq.domain.json.channels.info.ChannelInfosList;
import ch.psi.daq.domain.query.ChannelNameRequest;
import ch.psi.daq.domain.request.range.RequestRangePulseId;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

/**
 * Tests the {@link DaqController} implementation.
 */
public class QueryRestControllerChannelInfoTest extends AbstractDaqRestTest {

   private ObjectMapper objectMapper = new ObjectMapper();
   
   @Resource(name = DomainConfig.BEAN_NAME_BACKEND_DEFAULT)
   private Backend backend;

   @After
   public void tearDown() throws Exception {}

   @Test
   public void testChannelInfoQuery_01() throws Exception {
      ChannelNameRequest query = new ChannelNameRequest(
            new RequestRangePulseId(
                  100,
                  101),
                  backend.getKey() + "1", backend.getKey() + "2");

      String content = mapper.writeValueAsString(query);
      System.out.println(content);

      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .post(DomainConfig.PATH_CHANNELS_INFO)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      List<? extends ChannelInfos> infosList = objectMapper.readValue(response, ChannelInfosList.class);
      assertEquals(2, infosList.size());
      ChannelInfos cInfos = infosList.get(0);
      assertEquals(backend.getKey() + "1", cInfos.getChannel().getName());
      assertEquals(backend, cInfos.getChannel().getBackend());
      List<ChannelInfo> infos = cInfos.getChannelInfos().collect(Collectors.toList());
      assertEquals(2, infos.size());
      ChannelInfo info = infos.get(0);
      assertEquals(backend.getKey() + "1", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());
      info = infos.get(1);
      assertEquals(backend.getKey() + "1", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());

      cInfos = infosList.get(1);
      assertEquals(backend.getKey() + "2", cInfos.getChannel().getName());
      assertEquals(backend, cInfos.getChannel().getBackend());
      infos = cInfos.getChannelInfos().collect(Collectors.toList());
      assertEquals(2, infos.size());
      info = infos.get(0);
      assertEquals(backend.getKey() + "2", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());
      info = infos.get(1);
      assertEquals(backend.getKey() + "2", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());
   }
   
   @Test
   public void testChannelInfoQuery_02() throws Exception {
      ChannelNameRequest query = new ChannelNameRequest(
            new RequestRangePulseId(
                  100,
                  101),
                  backend.getKey() + "1", backend.getKey() + "2");
      query.setOrdering(Ordering.desc);

      String content = mapper.writeValueAsString(query);
      System.out.println(content);

      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .post(DomainConfig.PATH_CHANNELS_INFO)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      List<? extends ChannelInfos> infosList = objectMapper.readValue(response, ChannelInfosList.class);
      assertEquals(2, infosList.size());
      ChannelInfos cInfos = infosList.get(0);
      assertEquals(backend.getKey() + "1", cInfos.getChannel().getName());
      assertEquals(backend, cInfos.getChannel().getBackend());
      List<ChannelInfo> infos = cInfos.getChannelInfos().collect(Collectors.toList());
      assertEquals(2, infos.size());
      ChannelInfo info = infos.get(0);
      assertEquals(backend.getKey() + "1", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());
      info = infos.get(1);
      assertEquals(backend.getKey() + "1", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());

      cInfos = infosList.get(1);
      assertEquals(backend.getKey() + "2", cInfos.getChannel().getName());
      assertEquals(backend, cInfos.getChannel().getBackend());
      infos = cInfos.getChannelInfos().collect(Collectors.toList());
      assertEquals(2, infos.size());
      info = infos.get(0);
      assertEquals(backend.getKey() + "2", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());
      info = infos.get(1);
      assertEquals(backend.getKey() + "2", info.getChannel());
      assertEquals(backend, info.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), info.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), info.getPulseId());
      assertArrayEquals(new int[] {1}, info.getShape());
      assertEquals(Type.Int32.getKey(), info.getType());
   }
}
